@@ -15,6 +15,7 @@
 #include <AppStreamQt/component.h>
 
 #include <QAbstractItemModel>
+#include <QIcon>
 #include <QPixmap>
 
 #include <QCoroTask>
@@ -107,7 +108,7 @@ public:
     QString arch() const;
     QString branch() const;
     bool canExecute() const override;
-    QStringList categories() override;
+    bool hasCategory(const QString &category) const override;
     QString comment() override;
     quint64 downloadSize() const;
     QVariant icon() const override;
@@ -201,9 +202,7 @@ public:
         return m_appdata;
     }
 
-    QString contentRatingText() const override;
     QString contentRatingDescription() const override;
-    ContentIntensity contentRatingIntensity() const override;
     uint contentRatingMinimumAge() const override;
 
     QStringList topObjects() const override;
@@ -221,6 +220,11 @@ public:
         return m_toUpdate;
     }
 
+    [[nodiscard]] bool hasResolvedIcon() const override;
+    void resolveIcon() override;
+    QString verifiedMessage() const override;
+    bool updateNeedsAttention() override;
+
 Q_SIGNALS:
     void hasDataChanged();
     void propertyStateChanged(FlatpakResource::PropertyKind kind, FlatpakResource::PropertyState state);
@@ -229,6 +233,8 @@ Q_SIGNALS:
 private:
     void setEolReason(const QString &reason);
     void loadPermissions();
+    QVector<FlatpakPermission> readPermissions(const QString &file) const;
+    void reportSizeChange();
 
     const AppStream::Component m_appdata;
     FlatpakResource::Id m_id;
@@ -257,6 +263,8 @@ private:
     static const QStringList s_bottomObjects;
     QVector<QByteArray> m_toUpdate;
     bool m_stockIcon;
+    std::optional<QIcon> m_icon;
+    QString m_newPermissions;
 };
 
 inline uint qHash(const FlatpakResource::Id &key)
