@@ -7,15 +7,18 @@
 #include "KNSResource.h"
 #include "KNSBackend.h"
 #include <KLocalizedString>
-#include <KNSCore/Engine>
+#include <KNSCore/EngineBase>
+#include <KNSCore/Transaction>
 #include <KShell>
 #include <QProcess>
 #include <QRegularExpression>
 
 #include "ReviewsBackend/Rating.h"
-#include <knewstuff_version.h>
+#include <appstream/AppStreamUtils.h>
+#include <attica/provider.h>
+#include <utils.h>
 
-KNSResource::KNSResource(const KNSCore::EntryInternal &entry, QStringList categories, KNSBackend *parent)
+KNSResource::KNSResource(const KNSCore::Entry &entry, QStringList categories, KNSBackend *parent)
     : AbstractResource(parent)
     , m_categories(std::move(categories))
     , m_entry(entry)
@@ -29,17 +32,17 @@ KNSResource::~KNSResource() = default;
 AbstractResource::State KNSResource::state()
 {
     switch (m_entry.status()) {
-    case KNS3::Entry::Invalid:
+    case KNSCore::Entry::Invalid:
         return Broken;
-    case KNS3::Entry::Downloadable:
+    case KNSCore::Entry::Downloadable:
         return None;
-    case KNS3::Entry::Installed:
+    case KNSCore::Entry::Installed:
         return Installed;
-    case KNS3::Entry::Updateable:
+    case KNSCore::Entry::Updateable:
         return Upgradeable;
-    case KNS3::Entry::Deleted:
-    case KNS3::Entry::Installing:
-    case KNS3::Entry::Updating:
+    case KNSCore::Entry::Deleted:
+    case KNSCore::Entry::Installing:
+    case KNSCore::Entry::Updating:
         return None;
     }
     return None;
@@ -52,8 +55,8 @@ KNSBackend *KNSResource::knsBackend() const
 
 QVariant KNSResource::icon() const
 {
-    const QString thumbnail = m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall1);
-    return thumbnail.isEmpty() ? knsBackend()->iconName() : m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall1);
+    const QString thumbnail = m_entry.previewUrl(KNSCore::Entry::PreviewSmall1);
+    return thumbnail.isEmpty() ? knsBackend()->iconName() : m_entry.previewUrl(KNSCore::Entry::PreviewSmall1);
 }
 
 QString KNSResource::comment()
@@ -104,9 +107,9 @@ QString KNSResource::packageName() const
     return m_entry.uniqueId();
 }
 
-QStringList KNSResource::categories()
+bool KNSResource::hasCategory(const QString &category) const
 {
-    return m_categories;
+    return m_categories.contains(category);
 }
 
 QUrl KNSResource::homepage()
@@ -114,7 +117,7 @@ QUrl KNSResource::homepage()
     return m_entry.homepage();
 }
 
-void KNSResource::setEntry(const KNSCore::EntryInternal &entry)
+void KNSResource::setEntry(const KNSCore::Entry &entry)
 {
     const bool diff = entry.status() != m_lastStatus;
     m_entry = entry;
@@ -124,14 +127,14 @@ void KNSResource::setEntry(const KNSCore::EntryInternal &entry)
     }
 }
 
-KNSCore::EntryInternal KNSResource::entry() const
+KNSCore::Entry KNSResource::entry() const
 {
     return m_entry;
 }
 
 QJsonArray KNSResource::licenses()
 {
-    return {QJsonObject{{QStringLiteral("name"), m_entry.license()}, {QStringLiteral("url"), QString()}}};
+    return {{AppStreamUtils::license(m_entry.license())}};
 }
 
 quint64 KNSResource::size()
@@ -158,34 +161,46 @@ QString KNSResource::origin() const
     return m_entry.providerId();
 }
 
+QString KNSResource::displayOrigin() const
+{
+    if (auto providers = knsBackend()->engine()->atticaProviders(); !providers.isEmpty()) {
+        auto provider = providers.constFirst();
+        if (provider->name() == QLatin1String("api.kde-look.org")) {
+            return i18nc("The name of the KDE Store", "KDE Store");
+        }
+        return providers.constFirst()->name();
+    }
+    return QUrl(m_entry.providerId()).host();
+}
+
 QString KNSResource::section()
 {
     return m_entry.category();
 }
 
-static void appendIfValid(QList<QUrl> &list, const QUrl &value, const QUrl &fallback = {})
+static bool isAnimated(const QString &path)
 {
-    if (!list.contains(value)) {
-        if (value.isValid() && !value.isEmpty())
-            list << value;
-        else if (!fallback.isEmpty())
-            appendIfValid(list, fallback);
+    static const QVector<QLatin1String> s_extensions = {QLatin1String(".gif"), QLatin1String(".apng"), QLatin1String(".webp"), QLatin1String(".avif")};
+    return kContains(s_extensions, [path](const QLatin1String &postfix) {
+        return path.endsWith(postfix);
+    });
+}
+
+static void appendIfValid(Screenshots &list, const QUrl &thumbnail, const QUrl &screenshot)
+{
+    if (thumbnail.isEmpty() || screenshot.isEmpty()) {
+        return;
     }
+    list += {thumbnail, screenshot, isAnimated(thumbnail.path()), {}};
 }
 
 void KNSResource::fetchScreenshots()
 {
-    QList<QUrl> preview;
-    appendIfValid(preview, QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall1)));
-    appendIfValid(preview, QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall2)));
-    appendIfValid(preview, QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall3)));
-
-    QList<QUrl> screenshots;
-    appendIfValid(screenshots, QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewBig1)), QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall1)));
-    appendIfValid(screenshots, QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewBig2)), QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall2)));
-    appendIfValid(screenshots, QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewBig3)), QUrl(m_entry.previewUrl(KNSCore::EntryInternal::PreviewSmall3)));
-
-    Q_EMIT screenshotsFetched(preview, screenshots);
+    Screenshots ret;
+    appendIfValid(ret, QUrl(m_entry.previewUrl(KNSCore::Entry::PreviewSmall1)), QUrl(m_entry.previewUrl(KNSCore::Entry::PreviewBig1)));
+    appendIfValid(ret, QUrl(m_entry.previewUrl(KNSCore::Entry::PreviewSmall2)), QUrl(m_entry.previewUrl(KNSCore::Entry::PreviewBig2)));
+    appendIfValid(ret, QUrl(m_entry.previewUrl(KNSCore::Entry::PreviewSmall3)), QUrl(m_entry.previewUrl(KNSCore::Entry::PreviewBig3)));
+    Q_EMIT screenshotsFetched(ret);
 }
 
 void KNSResource::fetchChangelog()
@@ -200,7 +215,7 @@ QStringList KNSResource::extends() const
 
 QUrl KNSResource::url() const
 {
-    return QUrl(QStringLiteral("kns://") + knsBackend()->name() + QLatin1Char('/') + QUrl(m_entry.providerId()).host() + QLatin1Char('/') + m_entry.uniqueId());
+    return QUrl(QStringLiteral("kns://") + knsBackend()->name() + QLatin1Char('/') + m_entry.uniqueId());
 }
 
 bool KNSResource::canExecute() const
@@ -210,7 +225,7 @@ bool KNSResource::canExecute() const
 
 void KNSResource::invokeApplication() const
 {
-    knsBackend()->engine()->adoptEntry(m_entry);
+    KNSCore::Transaction::adopt(knsBackend()->engine(), m_entry);
 }
 
 QString KNSResource::executeLabel() const
@@ -239,18 +254,20 @@ QUrl KNSResource::donationURL()
     return QUrl(m_entry.donationLink());
 }
 
-Rating *KNSResource::ratingInstance()
+Rating KNSResource::rating() const
 {
-    if (!m_rating) {
+    if (!m_rating.has_value()) {
         const int noc = m_entry.numberOfComments();
         const int rating = m_entry.rating();
         Q_ASSERT(rating <= 100);
-        m_rating.reset(new Rating(packageName(), noc, rating / 10));
+        m_rating = Rating(packageName(), noc, rating / 10);
     }
-    return m_rating.data();
+    return m_rating.value();
 }
 
 QString KNSResource::author() const
 {
     return m_entry.author().name();
 }
+
+#include "moc_KNSResource.cpp"
